/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */
'use strict';

var RelayModernTestUtils = require("./RelayModernTestUtils");

var RelayTestSchema = require("./RelayTestSchema");

var areEqual = require("fbjs/lib/areEqual");

var invariant = require("fbjs/lib/invariant");
/* global jest */


var MAX_SIZE = 10;
var MAX_TTL = 5 * 60 * 1000; // 5 min

function mockInstanceMethod(object, key) {
  object[key] = jest.fn(object[key].bind(object));
}

function mockDisposableMethod(object, key) {
  var fn = object[key].bind(object);
  object[key] = jest.fn(function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var disposable = fn.apply(void 0, args);
    var dispose = jest.fn(function () {
      return disposable.dispose();
    });
    object[key].mock.dispose = dispose;
    return {
      dispose: dispose
    };
  });
  var mockClear = object[key].mockClear.bind(object[key]);

  object[key].mockClear = function () {
    mockClear();
    object[key].mock.dispose = null;
  };
}

function mockObservableMethod(object, key) {
  var fn = object[key].bind(object);
  object[key] = jest.fn(function () {
    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    return fn.apply(void 0, args)["do"]({
      start: function start(subscription) {
        object[key].mock.subscriptions.push(subscription);
      }
    });
  });
  object[key].mock.subscriptions = [];
  var mockClear = object[key].mockClear.bind(object[key]);

  object[key].mockClear = function () {
    mockClear();
    object[key].mock.subscriptions = [];
  };
}
/**
 * Creates an instance of the `Environment` interface defined in
 * RelayStoreTypes with a mocked network layer.
 *
 * Usage:
 *
 * ```
 * const environment = RelayModernMockEnvironment.createMockEnvironment();
 * ```
 *
 * Mock API:
 *
 * Helpers are available as `environment.mock.<helper>`:
 *
 * - `compile(text: string): {[queryName]: Query}`: Create a query.
 * - `isLoading(query, variables): boolean`: Determine whether the given query
 *   is currently being loaded (not yet rejected/resolved).
 * - `reject(query, error: Error): void`: Reject a query that has been fetched
 *   by the environment.
 * - `resolve(query, payload: PayloadData): void`: Resolve a query that has been
 *   fetched by the environment.
 */


function createMockEnvironment(options) {
  var _require = require("relay-runtime"),
      RecordSource = _require.RecordSource,
      Store = _require.Store,
      QueryResponseCache = _require.QueryResponseCache,
      Observable = _require.Observable,
      Environment = _require.Environment,
      Network = _require.Network;

  var schema = options && options.schema;
  var handlerProvider = options && options.handlerProvider;
  var source = new RecordSource();
  var store = new Store(source);
  var cache = new QueryResponseCache({
    size: MAX_SIZE,
    ttl: MAX_TTL
  }); // Mock the network layer

  var pendingRequests = [];

  var execute = function execute(request, variables, cacheConfig) {
    var id = request.id,
        text = request.text;
    var cacheID = id || text;
    var cachedPayload = null;

    if (!cacheConfig || !cacheConfig.force) {
      cachedPayload = cache.get(cacheID, variables);
    }

    if (cachedPayload !== null) {
      return Observable.from(cachedPayload);
    }

    var nextRequest = {
      request: request,
      variables: variables,
      cacheConfig: cacheConfig
    };
    pendingRequests = pendingRequests.concat([nextRequest]);
    return Observable.create(function (sink) {
      nextRequest.sink = sink;
      return function () {
        pendingRequests = pendingRequests.filter(function (pending) {
          return !areEqual(pending, nextRequest);
        });
      };
    });
  }; // The same request may be made by multiple query renderers


  function getRequests(request) {
    var foundRequests = pendingRequests.filter(function (pending) {
      return areEqual(pending.request, request.params);
    });
    !foundRequests.length ? process.env.NODE_ENV !== "production" ? invariant(false, 'MockEnvironment: Cannot respond to request, it has not been requested yet.') : invariant(false) : void 0;
    foundRequests.forEach(function (foundRequest) {
      !foundRequest.sink ? process.env.NODE_ENV !== "production" ? invariant(false, 'MockEnvironment: Cannot respond to `%s`, it has not been requested yet.', foundRequest.name) : invariant(false) : void 0;
    });
    return foundRequests;
  }

  function ensureValidPayload(payload) {
    !(typeof payload === 'object' && payload !== null && payload.hasOwnProperty('data')) ? process.env.NODE_ENV !== "production" ? invariant(false, 'MockEnvironment(): Expected payload to be an object with a `data` key.') : invariant(false) : void 0;
    return payload;
  }

  var cachePayload = function cachePayload(request, variables, payload) {
    var _request$params = request.params,
        id = _request$params.id,
        text = _request$params.text;
    var cacheID = id || text;
    cache.set(cacheID, variables, payload);
  };

  var clearCache = function clearCache() {
    cache.clear();
  }; // Helper to compile a query with the given schema (or the test schema by
  // default).


  var compile = function compile(text) {
    return RelayModernTestUtils.generateAndCompile(text, schema || RelayTestSchema);
  }; // Helper to determine if a given query/variables pair is pending


  var isLoading = function isLoading(request, variables, cacheConfig) {
    return pendingRequests.some(function (pending) {
      return areEqual(pending.request, request.params) && areEqual(pending.variables, variables) && areEqual(pending.cacheConfig, cacheConfig || {});
    });
  }; // Helpers to reject or resolve the payload for an individual request.


  var reject = function reject(request, error) {
    if (typeof error === 'string') {
      error = new Error(error);
    }

    getRequests(request).forEach(function (foundRequest) {
      return foundRequest.sink.error(error);
    });
  };

  var nextValue = function nextValue(request, payload) {
    getRequests(request).forEach(function (foundRequest) {
      var sink = foundRequest.sink;
      sink.next(ensureValidPayload(payload));
    });
  };

  var complete = function complete(request) {
    getRequests(request).forEach(function (foundRequest) {
      return foundRequest.sink.complete();
    });
  };

  var resolve = function resolve(request, payload) {
    getRequests(request).forEach(function (foundRequest) {
      var sink = foundRequest.sink;
      sink.next(ensureValidPayload(payload));
      sink.complete();
    });
  }; // Mock instance


  var environment = new Environment({
    configName: 'RelayModernMockEnvironment',
    handlerProvider: handlerProvider,
    network: Network.create(execute, execute),
    store: store
  }); // Mock all the functions with their original behavior

  mockDisposableMethod(environment, 'applyUpdate');
  mockInstanceMethod(environment, 'commitPayload');
  mockInstanceMethod(environment, 'getStore');
  mockInstanceMethod(environment, 'lookup');
  mockInstanceMethod(environment, 'check');
  mockDisposableMethod(environment, 'subscribe');
  mockDisposableMethod(environment, 'retain');
  mockObservableMethod(environment, 'execute');
  mockObservableMethod(environment, 'executeMutation');
  mockInstanceMethod(store, 'getSource');
  mockInstanceMethod(store, 'lookup');
  mockInstanceMethod(store, 'notify');
  mockInstanceMethod(store, 'publish');
  mockDisposableMethod(store, 'retain');
  mockDisposableMethod(store, 'subscribe');
  environment.mock = {
    cachePayload: cachePayload,
    clearCache: clearCache,
    compile: compile,
    isLoading: isLoading,
    reject: reject,
    resolve: resolve,
    nextValue: nextValue,
    complete: complete
  };

  environment.mockClear = function () {
    environment.applyUpdate.mockClear();
    environment.commitPayload.mockClear();
    environment.getStore.mockClear();
    environment.lookup.mockClear();
    environment.check.mockClear();
    environment.subscribe.mockClear();
    environment.retain.mockClear();
    environment.execute.mockClear();
    environment.executeMutation.mockClear();
    store.getSource.mockClear();
    store.lookup.mockClear();
    store.notify.mockClear();
    store.publish.mockClear();
    store.retain.mockClear();
    store.subscribe.mockClear();
    cache.clear();
    pendingRequests = [];
  };

  return environment;
}

module.exports = {
  createMockEnvironment: createMockEnvironment
};